/***********************************************************************
 *      This example program is an excerpt from the book:              *
 *                                                                     *
 *          X Window OSF/Motif Toolkit Programming (I)                 *
 *            - Programming Basics, Resource and Event -               *
 *                                                                     *
 *      Copyright 1994, Hideyuki Hayashi                               *
 *                                                                     *
 *      This program can be distributed without fee, provided          *
 *      that the above copyright notice appear in all copies.          *
 ***********************************************************************/

#include <Xm/XmAll.h>
static void ButtonCB();

/* Fallback Resources */
static String fallbacks[] = {
    "Sepinit.title:                  Simple Motif Example",
    "Sepinit*control.orientation:    HORIZONTAL",
    "Sepinit*fontList:               *-14-*:",
    "Sepinit*label.labelString:      File Name",
    "Sepinit*button.labelString:     Open",
    NULL
};

main(argc, argv)
    int  argc;
    char **argv;
{
    XtAppContext app_context;
    Display *display;
    Widget  toplevel, panel, control, label, text, button;
    Arg     al[20];
    int     ac;

    /* (1) Initialize X Toolkit */
    XtToolkitInitialize();

    /* (2) Create application context */
    app_context = XtCreateApplicationContext();

    /* Register language procedure (before opening a display) */
    XtSetLanguageProc(NULL, NULL, NULL);

    /* (3) Set fallback resource */
    XtAppSetFallbackResources(app_context, fallbacks);

    /* (4) Open and initialize the display */
    display = XtOpenDisplay(app_context, NULL,
                            NULL, "Sepinit", NULL, 0, &argc, argv);

    /* (5) Create toplevel shell */
    ac = 0;
    toplevel = XtAppCreateShell(NULL, "Sepinit", applicationShellWidgetClass,
                                display, al, ac);

    /* Create widgets as usual manner */
    ac = 0;
    panel = XmCreateRowColumn(toplevel, "panel", al, ac);
    XtManageChild(panel);
    ac = 0;
    control = XmCreateRowColumn(panel, "control", al, ac);
    XtManageChild(control);
    ac = 0;
    label = XmCreateLabel(control, "label", al, ac);
    XtManageChild(label);
    ac = 0;
    text = XmCreateTextField(control, "text", al, ac);
    XtManageChild(text);
    ac = 0;
    button = XmCreatePushButton(panel, "button", al, ac);
    XtManageChild(button);

    XtAddCallback(button, XmNactivateCallback, ButtonCB, text);

    XtRealizeWidget(toplevel);
    XtAppMainLoop(app_context);
}

static void ButtonCB(w, client_data, call_data)
    Widget    w;
    XtPointer client_data;
    XtPointer call_data;
{
    Widget text = (Widget) client_data;
    char *filename;

    filename = XmTextFieldGetString(text);
    printf("Entered file name: %s\n", filename);
    XtFree(filename);
}
