/***********************************************************************
 *      This example program is an excerpt from the book:              *
 *                                                                     *
 *          X Window OSF/Motif Toolkit Programming (I)                 *
 *            - Programming Basics, Resource and Event -               *
 *                                                                     *
 *      Copyright 1994, Hideyuki Hayashi                               *
 *                                                                     *
 *      This program can be distributed without fee, provided          *
 *      that the above copyright notice appear in all copies.          *
 ***********************************************************************/

#include <X11/Xatom.h>
#include <Xm/XmAll.h>

static GC gc;

static void InputEventCB(w, client_data, call_data)
    Widget    w;
    XtPointer client_data;
    XtPointer call_data;
{
    XmDrawingAreaCallbackStruct *cb = (XmDrawingAreaCallbackStruct *)call_data;
    Window window = (Window)client_data;
    XEvent event;
    Atom custom, clear;

    if (cb->event->type == ButtonPress) {
        /* If button1 is pressed, send the event to recvevent */
        if (cb->event->xbutton.button == Button1) {
            cb->event->xbutton.window = window;
            XSendEvent(XtDisplay(w), window, False, ButtonPressMask, cb->event);
        }
        /* If button3 is pressed, send ClientMessage to clear the window */
        else if (cb->event->xbutton.button == Button3) {
            custom = XmInternAtom(XtDisplay(w), "CUSTOM MESSAGE", False);
            clear = XmInternAtom(XtDisplay(w), "CLEAR WINDOW", False);
            event.type = ClientMessage;
            event.xany.display = XtDisplay(w);
            event.xany.window = window;
            event.xclient.message_type = custom;
            event.xclient.format = 32;
            event.xclient.data.l[0] = clear;
            XSendEvent(XtDisplay(w), window, False, NoEventMask, &event);
        }
    }
}

main(argc, argv)
    int  argc;
    char **argv;
{
    XtAppContext app_context;
    Widget  toplevel, bboard, canvas;
    Arg     al[20];
    int     ac;
    Window *window;  /* Window ID of the program recvevent */
    Atom    atom, type;
    int     format;
    unsigned long nitems, left;

    XtSetLanguageProc(NULL, NULL, NULL);

    ac = 0;
    toplevel = XtAppInitialize(&app_context, "Recvevent", NULL, 0,
                               &argc, argv, NULL, al, ac);

    ac = 0;
    bboard = XmCreateBulletinBoard(toplevel, "bboard", al, ac);
    XtManageChild(bboard);

    ac = 0;
    XtSetArg(al[ac], XmNwidth, 300); ac++;
    XtSetArg(al[ac], XmNheight, 300); ac++;
    canvas = XmCreateDrawingArea(bboard, "canvas", al, ac);
    XtManageChild(canvas);

    /* Get the window ID of recvevent */
    atom = XmInternAtom(XtDisplay(canvas), "CANVAS WINDOW", False);
    XGetWindowProperty(XtDisplay(canvas), RootWindowOfScreen(XtScreen(canvas)),
                       atom, 0, 4, False, XA_WINDOW, &type, &format,
                       &nitems, &left, (unsigned char **)&window);

    XtAddCallback(canvas, XmNinputCallback, InputEventCB, (XtPointer)*window);

    XtRealizeWidget(toplevel);

    gc = XCreateGC(XtDisplay(canvas), XtWindow(canvas), NULL, NULL);
    XSetForeground(XtDisplay(canvas), gc, BlackPixelOfScreen(XtScreen(canvas)));

    XtAppMainLoop(app_context);
}
