/***********************************************************************
 *      This example program is an excerpt from the book:              *
 *                                                                     *
 *          X Window OSF/Motif Toolkit Programming (I)                 *
 *            - Programming Basics, Resource and Event -               *
 *                                                                     *
 *      Copyright 1994, Hideyuki Hayashi                               *
 *                                                                     *
 *      This program can be distributed without fee, provided          *
 *      that the above copyright notice appear in all copies.          *
 ***********************************************************************/

#include <Xm/XmAll.h>

/* Input callback function called when input occurs */
static void InputCB(client_data, source, id)
    XtPointer  client_data; /* Client data passed when registered */
    int       *source;      /* File descriptor to be inputted */
    XtInputId *id;          /* ID of this input callback */
{
    Widget   label = (Widget)client_data;
    char     buff[BUFSIZ];
    int      len;
    XmString labelstr;

    len = read(*source, buff, BUFSIZ); /* Input from the file descriptor */
    if (len < 1) {           /* If the read fails, */
        XtRemoveInput(*id);  /* delete this input callback */
        return;              /* so that it is never called. */
    }
    if (buff[len-1] == '\n') buff[len-1] = NULL;  /* Ignore the last new line */

    /* Create compound string and show on the label */
    labelstr = XmStringCreateLocalized(buff);
    XtVaSetValues(label, XmNlabelString, labelstr, NULL);
    XmStringFree(labelstr);  /* Free the unnecessary compound string */
}

main(argc, argv)
    int  argc;
    char **argv;
{
    XtAppContext app_context;
    Widget  toplevel, panel, field, title, label, button;
    Arg     al[20];
    int     ac;
    FILE   *fp;
    int     fd;

    XtSetLanguageProc(NULL, NULL, NULL);
    ac = 0;
    toplevel = XtAppInitialize(&app_context, "Inputcb", NULL, 0,
                               &argc, argv, NULL, al, ac);
    ac = 0;
    panel = XmCreateRowColumn(toplevel, "panel", al, ac);
    XtManageChild(panel);
    ac = 0;
    field = XmCreateRowColumn(panel, "field", al, ac);
    XtManageChild(field);
    ac = 0;
    title = XmCreateLabel(field, "title", al, ac);
    XtManageChild(title);
    ac = 0;
    label = XmCreateLabel(field, "label", al, ac);
    XtManageChild(label);
    ac = 0;
    button = XmCreatePushButton(panel, "button", al, ac);
    XtManageChild(button);

    fp = popen("printrandom", "r");  /* Invoke other process by pipe */
    if (fp == NULL) {
        fprintf(stderr, "Cannot start sub program.\n");
        exit(1);
    }

    fd = fileno(fp);    /* File descriptor of pipe (fp) */
    /* Register the input callback to application context */
    XtAppAddInput(app_context, fd, (XtPointer)XtInputReadMask, InputCB, label);

    XtRealizeWidget(toplevel);
    XtAppMainLoop(app_context);
}
