/***********************************************************************
 *      This example program is an excerpt from the book:              *
 *                                                                     *
 *          X Window OSF/Motif Toolkit Programming (I)                 *
 *            - Programming Basics, Resource and Event -               *
 *                                                                     *
 *      Copyright 1994, Hideyuki Hayashi                               *
 *                                                                     *
 *      This program can be distributed without fee, provided          *
 *      that the above copyright notice appear in all copies.          *
 ***********************************************************************/

#include <Xm/XmAll.h>

#define SCALE   5    /* scaling */

/* (3B) Called when pointer is moved */
static void RadarEH(w, client_data, event, dispatch)
    Widget     w;
    XtPointer  client_data;
    XEvent    *event;
    Boolean   *dispatch;
{
    GC gc = (GC)client_data;  /* Graphics Context necessary for drawing */

         /* Is the event of pointer motion ? */
    if ( event->type == MotionNotify &&
         /* Is the event on root window */
         event->xany.window == RootWindowOfScreen(XtScreen(w)) ) {

        /* once clear the window */
        XClearWindow(XtDisplay(w), XtWindow(w));
        /* draw a circle at the location of 1/SCALE of screen */
        XFillArc(XtDisplay(w), XtWindow(w), gc,
                 event->xmotion.x / SCALE - 5, event->xmotion.y / SCALE - 5,
                 11, 11, 0, 360*64);
    }
}

main(argc, argv)
    int  argc;
    char **argv;
{
    XtAppContext app_context;
    Widget toplevel, canvas;
    Arg    al[20];
    int    ac;
    int    screen_w, screen_h;
    GC     gc;

    XtSetLanguageProc(NULL, NULL, NULL);

    ac = 0;
    toplevel = XtOpenApplication(&app_context, "Windowcb", NULL, 0,
                        &argc, argv, NULL, sessionShellWidgetClass, al, ac);

    /* Get the size of the screen */
    screen_w = WidthOfScreen(XtScreen(toplevel));
    screen_h = HeightOfScreen(XtScreen(toplevel));

    ac = 0;
    /* Set the size of drawing area to the 1/SCALE of screen */
    XtSetArg(al[ac], XmNwidth,  screen_w / SCALE); ac++;
    XtSetArg(al[ac], XmNheight, screen_h / SCALE); ac++;
    canvas = XmCreateDrawingArea(toplevel, "canvas", al, ac);
    XtManageChild(canvas);

    /* (1) Preparation for receiving event */
    /*     Require pointer motion event on root window */
    XSelectInput(XtDisplay(toplevel), RootWindowOfScreen(XtScreen(toplevel)),
                 PointerMotionMask);

    /* (2) Register root window to the widget */
    XtRegisterDrawable(XtDisplay(toplevel),
                       RootWindowOfScreen(XtScreen(toplevel)), canvas);

    /* Check the registration by XtWindowToWidget() */
    printf("Root window is registered to widget `%s'\n",
           XtName(XtWindowToWidget(XtDisplay(toplevel),
                                   RootWindowOfScreen(XtScreen(toplevel)))));

    XtRealizeWidget(toplevel);

    gc = XCreateGC(XtDisplay(canvas), XtWindow(canvas), NULL, NULL);
    XSetForeground(XtDisplay(canvas), gc, BlackPixelOfScreen(XtScreen(canvas)));

    /* (3A) Register event handler on drawing area */
    /*    (this event handler is called also when pointer is moved on root window) */
    XtAddEventHandler(canvas, PointerMotionMask, False, RadarEH, gc);

    XtAppMainLoop(app_context);
}
